/*
 * Travel.java
 *
 * Created on August 10, 2006, 3:50 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package ee5travel.ws.module;

import ee5travel.entities.Person;
import ee5travel.entities.Trip;
import ee5travel.entities.Triptype;
import java.util.List;
import javax.ejb.Stateless;
import javax.jws.WebService;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

/**
 * <p>Business logic for the Travel Center application, exposed as a
 * web service.  Because we are encapsulated in a stateless session bean,
 * we also do not need to worry about thread safety.</p>
 */

@Stateless()
@WebService()
public class Travel {
    
    /* Sample Web Service Operation */
    
    @WebMethod(operationName="sample_operation")
    public String operation(@WebParam(name="param_name") String param) {
        // implement the web service operation here
        return param;
    }
    

    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The entity manager we will use for accessing the database.</p>
     */
    @PersistenceContext(unitName="travel")
    EntityManager em;


    // ----------------------------------------------------- Web Service Methods


    /**
     * <p>Read-only property to return all {@link Person} instances.
     * You would not usually see such a method in a real application,
     * but we know that our demo database has a very constrained
     * set of rows.</p>
     */
    @WebMethod
    public List<Person> getPersons() {

        try {
            Query query = em.createQuery("SELECT p FROM Person p");
            return query.getResultList();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        }

    }


    /**
     * <p>Read-only property to return all {@link Triptype} instances.
     */
    @WebMethod
    public List<Triptype> getTriptypes() {

        try {
            Query query = em.createQuery("SELECT t FROM Triptype t");
            return query.getResultList();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        }

    }


    /**
     * <p>Add a new {@link Person} instance.</p>
     *
     * @param person The {@link Person} instance to be added
     *
     * @return The updated {@link Person} instance
     */
    @WebMethod
    public Person addPerson(@WebParam(name="person") Person person) {

        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            em.persist(person);
            et.commit();
            return person;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
        }

    }


    /**
     * <p>Add a new {@link Trip} for the specified {@link Person}.</p>
     *
     * @param person The {@link Person} for whom to add a new {@link Trip}
     * @param trip The {@link Trip} to be added
     *
     * @return The updated {@link Trip} instance
     */
    @WebMethod
    public Trip addTrip(@WebParam(name="person") Person person,
                        @WebParam(name="trip") Trip trip) {

        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            person = em.find(Person.class, person.getPersonid());
            trip.setPersonid(person);
            em.persist(trip);
            person.getTripCollection().add(trip);
            et.commit();
            return trip;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
        }

    }


    /**
     * <p>Find an existing {@link Person} instance by id.</p>
     *
     * @return The found {@link Person} instance, if any;
     *  otherwise <code>null</code>
     */
    @WebMethod
    public Person findPersonById(@WebParam(name="id") int id) {

        try {
            return em.find(Person.class, id);
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        }

    }


    /**
     * <p>Find an existing {@link Trip} instance by id.</p>
     *
     * @return The found {@link Trip} instance, if any;
     *  otherwise <code>null</code>
     */
    @WebMethod
    public Trip findTripById(@WebParam(name="id") int id) {

        try {
            return em.find(Trip.class, id);
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        }

    }


    /**
     * <p>Return a list of {@link Trip}s for the specified
     * {@link Person}.</p>
     *
     * @param person {@link Person} for which to find trips
     */
    @WebMethod
    public List<Trip> findTripsByPerson(@WebParam(name="person") Person person) {

        try {
            Query query = em.createQuery("SELECT t FROM Trip t WHERE t.personid = :personid");
            query.setParameter("personid", person.getPersonid());
            return query.getResultList();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        }

    }


    /**
     * <p>Find an existing {@link Triptype} instance by id.</p>
     *
     * @return The found {@link Triptype} instance, if any;
     *  otherwise <code>null</code>
     */
    @WebMethod
    public Triptype findTriptypeById(@WebParam(name="id") int id) {

        try {
            return em.find(Triptype.class, id);
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        }

    }


    /**
     * <p>Remove an existing {@link Person} instance.</p>
     *
     * @param person The {@link Person} instance to be removed
     */
    @WebMethod
    public void removePerson(@WebParam(name="person") Person person) {

        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            person = em.merge(person);
            em.remove(person);
            et.commit();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
        }

    }


    /**
     * <p>Remove an existing {@link Trip} instance.</p>
     *
     * @param trip The {@link Trip} instance to be removed
     */
    @WebMethod
    public void removeTrip(@WebParam(name="trip") Trip trip) {

        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            trip = em.merge(trip);
            Person person = em.find(Person.class, trip.getPersonid().getPersonid());
            person.getTripCollection().remove(trip);
            em.remove(trip);
            et.commit();
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
        }

    }


    /**
     * <p>Update an existing {@link Person} instance.</p>
     *
     * @param person The {@link Person} instance to be updated
     *
     * @return The updated {@link Person} instance
     */
    @WebMethod
    public Person updatePerson(@WebParam(name="peson") Person person) {

        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            person = em.merge(person);
            et.commit();
            return person;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
        }

    }


    /**
     * <p>Update an existing {@link Trip} instance.</p>
     *
     * @param trip The {@link Trip} instance to be updated
     *
     * @return The updated {@link Trip} instance
     */
    @WebMethod
    public Trip updateTrip(@WebParam(name="trip") Trip trip) {

        EntityTransaction et = null;
        try {
            et = em.getTransaction();
            et.begin();
            trip = em.merge(trip);
            et.commit();
            return trip;
        } catch (PersistenceException e) {
            throw e;
        } catch (Exception e) {
            throw new PersistenceException(e);
        } finally {
            if ((et != null) && et.isActive()) {
                et.rollback();
            }
        }

    }


}
